package CicleCRR;

import Case.ICase;
import CaseBase.ICaseBase;
import RetrieveReuse.Retrieve;
import RetrieveReuse.Reuse;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 *
 */
public class CRR implements ICRR{
    /**
     * Mètode per executar l'algorisme CRR
     * @param caseBase
     * @param dataset
     * @param fileTrain
     * @return 
     */
    @Override
    public Collection<ICase> executeCRR(ICaseBase caseBase, String dataset, String fileTrain){		
        double[] minAttributeValues = caseBase.getMinAttributeValues();
        double[] maxAttributeValues = caseBase.getMaxAttributeValues();
        Retrieve retrieve = new Retrieve();
        Reuse reuse = new Reuse();
        int numK = 3;
        CoverageSet coverage = new CoverageSet();
        
        coverage.createCoverageSet(numK, caseBase, reuse, retrieve, minAttributeValues, maxAttributeValues);
        
        Map<String,Integer> casosSizeCoverage = new HashMap<String,Integer>();
        
        /*
        * Conservative Redundancy Removal(CRR) Algorithm:
        * T, Training Set
        * 
        * Build case-base competence model
        * For each c in T
        *              CSet(c) = Coverage Set of c
        * End-For
        * 
        * Remove redundant cases from case-base
        * ESet = {}, (Edited Set)
        * TSet = T sorted in ascending order of CSet(c) size
        * c = first case in TSet
        * 
        * While TSet != {}
        *              ESet = ESet + {c}
        *              TSet = TSet – CSet(c)
        *              c = next case in TSet
        * End-While
        * 
        * Return TSet
        */
        
        for(ICase c: caseBase.trainCases()){
            casosSizeCoverage.put(c.getName(), c.getCoverageSet().size());
        }
        casosSizeCoverage = this.sortHashMapByValues(casosSizeCoverage);

        //Crear el conjunt de casos ordenats per size de Coverage
        Collection <ICase> newTrainingSet = new ArrayList<>();
        List<String> mapKeys = new ArrayList<>(casosSizeCoverage.keySet());
        this.sortedByLiabilitySize(newTrainingSet, mapKeys, caseBase);
        
        //copiem el trainingset per poder esborrar i afegir casos sense problemes
        Collection<ICase> copySortSet = new ArrayList<>();
        Collection<ICase> editedSet = new ArrayList<>();
        Collection<ICase> removeCases = new ArrayList<>();
        for(ICase c: newTrainingSet){
            copySortSet.add(c);
        }
        
         // Use iterator
        Iterator itr = newTrainingSet.iterator();
        while(itr.hasNext() && !copySortSet.isEmpty()) {
           ICase cas = (ICase) itr.next();
           editedSet.add(cas);
           
           for(ICase remove: cas.getCoverageSet()){
               copySortSet.remove(remove);
               removeCases.add(remove);
           }
           
        }
     
        return removeCases;
    }
    
    /**
     * @param newTrainingSet
     * @param mapKeys
     * @param caseBase 
     */
    @Override
    public void sortedByLiabilitySize(Collection<ICase> newTrainingSet, List<String> mapKeys, ICaseBase caseBase){
        //Creem el conjunt de Casos ordenats
        for (String searchCase : mapKeys) {
            boolean trobat = false;
            
            Iterator itr = caseBase.trainCases().iterator();
            while(itr.hasNext() && !trobat) {
                ICase element = (ICase) itr.next();
                if(element.getName().equals(searchCase)){
                    newTrainingSet.add(element);
                    trobat = true;
                }
            }
        }
        
    }
    
    /**
     * @param passedMap
     * @return 
     */
    @Override
    public LinkedHashMap<String, Integer> sortHashMapByValues(Map<String, Integer> passedMap) {
        List<String> mapKeys = new ArrayList<>(passedMap.keySet());
        List<Integer> mapValues = new ArrayList<>(passedMap.values());
        Collections.sort(mapValues);
        Collections.sort(mapKeys);

        LinkedHashMap<String, Integer> sortedMap = new LinkedHashMap<>();

        Iterator<Integer> valueIt = mapValues.iterator();
        while (valueIt.hasNext()) {
            Integer val = valueIt.next();
            Iterator<String> keyIt = mapKeys.iterator();

            while (keyIt.hasNext()) {
                String key = keyIt.next();
                Integer comp1 = passedMap.get(key);
                Integer comp2 = val;

                if (comp1.equals(comp2)) {
                    keyIt.remove();
                    sortedMap.put(key, val);
                    break;
                }
            }
        }
        return sortedMap;
    }
}
