package CicleRENN;

import Case.ICase;
import CaseBase.ICaseBase;
import java.util.ArrayList;
import java.util.Collection;
import RetrieveReuse.*;

/**
 * Classe principal de l'algorisme RENN.
 */
public class RENN implements IRENN{
    /**
     * Mètode per executar l'algorisme RENN
     * @param caseBase
     * @param dataset
     * @param fileTrain
     * @param trainCases
     * @return 
     */
    @Override
    public Collection<ICase> executeRENN(ICaseBase caseBase, String dataset, String fileTrain, Collection<ICase> trainCases) {
        Retrieve retrieve = new Retrieve();
        Reuse reuse = new Reuse();
        //copia del training set per no modificar-ho        
        Collection <ICase> copyTrainingSet = new ArrayList<ICase>();
        Collection <ICase> removeCases = new ArrayList<ICase>();
        
        for(ICase c : caseBase.trainCases()){
            copyTrainingSet.add(c);
        }
        
        boolean changes = true; //quan aixo sigui false vol dir que no hi han mes canvis i sortirem del while
        //fan falta els min i els max? exactament per a que serveixen???
        double[] minAttributeValues = caseBase.getMinAttributeValues();
        double[] maxAttributeValues = caseBase.getMaxAttributeValues();	
        ArrayList<ICase> retrievedCases = new ArrayList<ICase>();
        
        /* RENN Algorithm:
        *
        * T: Training Set
        * 
        * Repeat
        *              changes = false
        *              For all x E T do
        *                      If x does not agree with the majority of its NN
        *                              T = T - {x}
        *                              changes = true
        *                      End-If
        *              End-For
        * Until not changes
        *
        * Return T     
        */
        
        while(changes){
            changes = false;
            for(ICase c : copyTrainingSet){
                if(!c.isClassified()){
                    c.setClassifiedTrue();
                    //If c does not agree with the majority of its NN
                    retrievedCases = retrieve.retrieveCases(c, copyTrainingSet, minAttributeValues, maxAttributeValues,3);
                    String classe = reuse.reviseCases(retrievedCases);
                    
                    boolean classified = false; //centinela trobat
                    if (c.getAttributeClass().equals(classe)) {					
                        classified = true; // Hem classificat el cas incorrectament ==> missclassified = true						
                    }

                    if(!classified){
                        copyTrainingSet.remove(c);
                        removeCases.add(c);
                        c.setNoise();
                        changes = true;
                        break;
                    }
                }
            }
        }
        
        return removeCases;
        
    }
    
}
