package RetrieveReuse;

import Attribute.IAttribute;
import Case.ICase;
import DistanceFunction.DistanceEuclidean;
import DistanceFunction.IDistanceFunction;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * Classe principal.
 */
public class Retrieve implements IRetrieve{
    /**
     * Mètode per ordenar un Map
     * @param passedMap
     * @return 
     */
    @Override
    public LinkedHashMap<String, Double> sortHashMapByValuesDoubles(Map<String, Double> passedMap){
        List<String> mapKeys = new ArrayList<>(passedMap.keySet());
        List<Double> mapValues = new ArrayList<>(passedMap.values());
        Collections.sort(mapValues);
        Collections.sort(mapKeys);

        LinkedHashMap <String, Double> sortedMap = new LinkedHashMap<>();

        Iterator<Double> valueIt = mapValues.iterator();
        while (valueIt.hasNext()) {
            Double val = valueIt.next();
            Iterator<String> keyIt = mapKeys.iterator();

            while (keyIt.hasNext()) {
                String key = keyIt.next();
                Double comp1 = passedMap.get(key);
                Double comp2 = val;

                if (comp1.equals(comp2)) {
                    keyIt.remove();
                    sortedMap.put(key, val);
                    break;
                }
            }
        }
        return sortedMap;
    }
    
    /**
     * Mètode que retorna uns determinats casos
     * @param testCase
     * @param trainCases
     * @param minAttributeValues
     * @param maxAttributeValues
     * @param numK
     * @return 
     */
    @Override
    public ArrayList<ICase> retrieveCases(ICase testCase, Collection<ICase> trainCases, double[] minAttributeValues, double[] maxAttributeValues, int numK){
        ArrayList<ICase> retrievedCases = new ArrayList<>();
        Map<String, Double> casesDistances = new HashMap<>();

        IDistanceFunction distanceFunction = new DistanceEuclidean();
        Collection<IAttribute> iTestAttr = testCase.getIAttributes();
        double dist = 0d;
        
        // Per cada cas de la base de casos
        //abans en comptes de caseBase.trainCases() hi havia el trainCases que ens passen per parametre
        for (ICase c : trainCases) {		
            if(!c.getName().equals(testCase.getName())){
                // Agafem els atributs del cas		
                Collection<IAttribute> iTrainAttr = c.getIAttributes();			
                // Calculem la distancia					
                dist = distanceFunction.calculateDistance2(trainCases, iTestAttr, iTrainAttr, minAttributeValues, maxAttributeValues);
                // Guardem la distancia en un array
                casesDistances.put(c.getName(), dist);
            }
            
        }
        
        casesDistances = this.sortHashMapByValuesDoubles((HashMap<String, Double>) casesDistances);

        
        List<String> keysCerca = new ArrayList<>(casesDistances.keySet());
        ArrayList<String> casosRetrieve = new ArrayList<String>();
        
        for(int i=0; i<numK; i++){
            if(keysCerca.get(i)!=null){
                casosRetrieve.add(keysCerca.get(i));
            }
        }
        
        int k = 0;
        while(k<numK){
            for(ICase c: trainCases){
                if(c.getName().equals(casosRetrieve.get(k))){
                    k++;
                    retrievedCases.add(c);
                    break;
                }
            }
        }
        
        return retrievedCases;
        
    }
}
